@@ -152,13 +152,14 @@ extern int errno;
 #endif
 
 #ifndef PATH_MAX
-# ifndef POSIX
+# ifdef MAXPATHLEN
 #  define PATH_MAX      MAXPATHLEN
+# else
+/* Some systems (HURD) have fully dynamic pathnames with no maximum.
+   Ideally we'd support this but it will take some work.  */
+#  define PATH_MAX      4096
 # endif
 #endif
-#ifndef MAXPATHLEN
-# define MAXPATHLEN 1024
-#endif
 
 #ifdef  PATH_MAX
 # define GET_PATH_MAX   PATH_MAX
@@ -494,6 +495,8 @@ extern struct rlimit stack_limit;
 
 /* Number of characters in a string constant.  Does NOT include the \0 byte.  */
 #define CSTRLEN(_s)           (sizeof (_s)-1)
+
+/* Only usable when NOT calling a macro: only use it for local functions.  */
 #define STRING_SIZE_TUPLE(_s) (_s), CSTRLEN(_s)
 
 /* The number of bytes needed to represent the largest signed and unsigned
@@ -657,8 +660,12 @@ int unload_file (const char *name);
 #ifdef MAKE_MAINTAINER_MODE
 # define SPIN(_s) spin (_s)
 void spin (const char* suffix);
+# define DBG(_f) dbg _f
+void dbg (const char *fmt, ...);
 #else
 # define SPIN(_s)
+/* Never put this code into Git or a release.  */
+# define DBG(_f) compile-error
 #endif
 
 /* We omit these declarations on non-POSIX systems which define _POSIX_VERSION,
