@@ -530,6 +530,22 @@ spin (const char* type)
     }
 }
 
+void
+dbg (const char *fmt, ...)
+{
+  FILE *fp = fopen ("/tmp/gmkdebug.log", "a+");
+  va_list args;
+  char buf[4096];
+
+  va_start (args, fmt);
+  vsprintf (buf, fmt, args);
+  va_end (args);
+
+  fprintf(fp, "%u: %s\n", (unsigned) make_pid (), buf);
+  fflush (fp);
+  fclose (fp);
+}
+
 #endif
 
 
@@ -552,7 +568,6 @@ umask (mode_t mask)
 }
 #endif
 
-#define MAKE_TMPDIR         "MAKE_TMPDIR"
 #ifdef VMS
 # define DEFAULT_TMPFILE    "sys$scratch:gnv$make_cmdXXXXXX.com"
 #else
@@ -566,13 +581,36 @@ get_tmpdir ()
 
   if (!tmpdir)
     {
-      if (((tmpdir = getenv (MAKE_TMPDIR)) == NULL || *tmpdir == '\0')
 #if defined (__MSDOS__) || defined (WINDOWS32) || defined (__EMX__)
-          && ((tmpdir = getenv ("TMP")) == NULL || *tmpdir == '\0')
-          && ((tmpdir = getenv ("TEMP")) == NULL || *tmpdir == '\0')
+# define TMP_EXTRAS   "TMP", "TEMP",
+#else
+# define TMP_EXTRAS
 #endif
-          && ((tmpdir = getenv ("TMPDIR")) == NULL || *tmpdir == '\0'))
-        tmpdir = DEFAULT_TMPDIR;
+      const char *tlist[] = { "MAKE_TMPDIR", "TMPDIR", TMP_EXTRAS NULL };
+      const char **tp;
+      unsigned int found = 0;
+
+      for (tp = tlist; *tp; ++tp)
+        if ((tmpdir = getenv (*tp)) && *tmpdir != '\0')
+          {
+            struct stat st;
+            int r;
+            found = 1;
+            EINTRLOOP(r, stat (tmpdir, &st));
+            if (r < 0)
+              OSSS (error, NILF,
+                    _("%s value %s: %s"), *tp, tmpdir, strerror (errno));
+            else if (! S_ISDIR (st.st_mode))
+              OSS (error, NILF,
+                   _("%s value %s: not a directory"), *tp, tmpdir);
+            else
+              return tmpdir;
+          }
+
+      tmpdir = DEFAULT_TMPDIR;
+
+      if (found)
+        OS (error, NILF, _("using default temporary directory '%s'"), tmpdir);
     }
 
   return tmpdir;
@@ -630,6 +668,7 @@ get_tmpfd (char **name)
 {
   int fd = -1;
   char *tmpnm;
+  mode_t mask;
 
   /* If there's an os-specific way to get an anoymous temp file use it.  */
   if (!name)
@@ -639,6 +678,10 @@ get_tmpfd (char **name)
         return fd;
     }
 
+  /* Preserve the current umask, and set a restrictive one for temp files.
+     Only really needed for mkstemp() but won't hurt for the open method.  */
+  mask = umask (0077);
+
 #if defined(HAVE_MKSTEMP)
   tmpnm = get_tmptemplate ();
 
@@ -666,6 +709,8 @@ get_tmpfd (char **name)
       free (tmpnm);
     }
 
+  umask (mask);
+
   return fd;
 }
 
@@ -769,11 +814,11 @@ get_path_max (void)
 
   if (value == 0)
     {
-      long int x = pathconf ("/", _PC_PATH_MAX);
+      long x = pathconf ("/", _PC_PATH_MAX);
       if (x > 0)
-        value = x;
+        value = (unsigned int) x;
       else
-        return MAXPATHLEN;
+        value = PATH_MAX;
     }
 
   return value;
